library(rstan)
library(shinystan)


teams <- as.vector (unlist (read.table ("euroTeams.txt", header=FALSE)))
nteams <- length(teams)

prior_score<- rev(1:nteams)
prior_score<-(prior_score - mean(prior_score))/(2*sd(prior_score))


prior_score_att <- rev(1:nteams)
prior_score_def <- rev(1:nteams)
priors<-read.csv2("priorTeams.csv", header=F)
prior_score_att<- (as.vector(priors[[2]])-mean(as.vector(priors[[2]]))) /(1.5*sd(as.vector(priors[[2]])))
prior_score_def<- (as.vector(priors[[3]])-mean(as.vector(priors[[3]]))) /(1.5*sd(as.vector(priors[[3]])))



data2016 <- read.csv2("euro2016.csv", header=FALSE)
ngames <- nrow (data2016)

team1 <- match (as.vector(data2016[[1]]), teams)
score1 <- as.vector(data2016[[2]])
team2 <- match (as.vector(data2016[[3]]), teams)
score2 <- as.vector(data2016[[4]])
K<-4
type<-as.vector(data2016[[5]])
type_fac<-as.factor(type)
type_num<-as.numeric(type_fac)




#round of 16

ottavi_veri<-read.csv2("ottavi_veri.csv", header=FALSE)

team1_prev<-match (as.vector(ottavi_veri[[1]]), teams)
team2_prev<-match (as.vector(ottavi_veri[[2]]), teams)
ngames_prev<-nrow(ottavi_veri)


data <- c("nteams","ngames","team1","score1","team2","score2", "ngames_prev",
          "team1_prev", "team2_prev", "prior_score_att", "prior_score_def", 
          "prior_score", "type_num")


fit_euroMatt_Round16 <- stan("euro2016MattRound16.stan", data=data, chains=4, iter=2000, cores=4)
launch_shinystan(fit_euroMatt_Round16)
print(fit_euroMatt_Round16)
sims_Round16<-extract(fit_euroMatt_Round16)




colVars <- function(a) {n <- dim(a)[[1]]; c <- dim(a)[[2]]; return(.colMeans(((a - matrix(.colMeans(a, n, c), nrow = n, ncol = c, byrow = TRUE)) ^ 2), n, c) * n / (n - 1))}

#attack

sims_Round16<-extract(fit_euroMatt_Round16)
att_sims <- sims$att
att_hat <- colMeans(att_sims)
att_se <- sqrt(colVars(att_sims))
library ("arm")
#pdf("euro2016.pdf", height=500, width=500)
coefplot (rev(att_hat), rev(att_se), CI=1, varnames=rev(teams), main="Attack quality (estimate +/- 1 s.e.)\n", cex.var=.9, mar=c(0,4,5.1,2), xlim=c(-1,1))
dev.copy2pdf(file="euro2016Attackbars.pdf")

#defense

sims_Round16<-extract(fit_euroMatt_Round16)
def_sims <- sims$def
def_hat <- colMeans(def_sims)
def_se <- sqrt(colVars(def_sims))
library ("arm")
#pdf("euro2016.pdf", height=500, width=500)
coefplot (rev(def_hat), rev(def_se), CI=1, varnames=rev(teams), main="Defense quality (estimate +/- 1 s.e.)\n", cex.var=.9, mar=c(0,4,5.1,2), xlim=c(-2,2))
 dev.copy2pdf(file="euro2016Defensebars.pdf")
 
 #games differentials
 
 valori<-apply(sims$diff_rep,2,median)
 sigma_y_sims <- sims$theta
 #interval_975 <-median(qpois(0.975, sims$theta)*sigma_y_sims)
 
 #lower <-valori - interval_975
 #upper <- valori + interval_975
 lower<-apply(sims$diff_rep,2,function(x) quantile(x,0.025))
 upper<-apply(sims$diff_rep,2,function(x) quantile(x,0.975))
 
 #png ("worldcup2.png", height=1000, width=500)
 coefplot (rev(score1[1:30] - score2[1:30]), sds=rep(0, 30),
           lower.conf.bounds=rev(lower[1:30]), upper.conf.bounds=rev(upper[1:30]), 
           varnames=rev(paste(teams[team1][1:30], "vs.", teams[team2][1:30])),
           main="Game score differentials\ncompared to 95% predictive interval from model\n",
           mar=c(0,7,6,2))




###PREDICTIONS Round of 16



sims_Round16<-extract(fit_euroMatt_Round16)
ris1<-sims_Round16$score1_prev
colMeans(ris1)
gol1<-apply(ris1,2,median)
ris2<-sims_Round16$score2_prev
colMeans(ris2)
gol2<-apply(ris2,2,median)
apply(sims_Round16$diff_prev,2,median)

cbind(gol1,gol2)


#grafici

par(mfrow=c(2,4))
for (i in 1:4){
  
  H<-hist(sims_Round16$score1_prev[,i], col="red", prob=T, xlab=teams[team1_prev[i]], main=paste("Game", i))
  H2<-hist(sims_Round16$score2_prev[,i], col="green", xlab=teams[team2_prev[i]], prob=T, main=paste("Game", i) )
  
}

par(mfrow=c(2,4))
for (i in 5:8){
  
  H<-hist(sims_Round16$score1_prev[,i], col="red", prob=T, xlab=teams[team1_prev[i]], main=paste("Game", i))
  H2<-hist(sims_Round16$score2_prev[,i], col="green", xlab=teams[team2_prev[i]], prob=T, main=paste("Game", i) )
  
}




