data{
  int nteams;
  int ngames;
  int team1[ngames];
  int team2[ngames];
  int score1[ngames];
  int score2[ngames];
  int ngames_prev;
  int team1_prev[ngames_prev];
  int team2_prev[ngames_prev];
  vector[nteams] prior_score_att;
  vector[nteams] prior_score_def;
  vector[nteams] prior_score;
  int type_num[ngames];
}
transformed data{
  int diff[ngames];
  for (n in 1:ngames){
    diff[n]<-score1[n]-score2[n];
  }
}
parameters{
  real b;
  vector[nteams] e_att;
  vector[nteams] e_def;
  real<lower=0> sigma_att;
  real<lower=0> sigma_def;
  real mu;
  real mu_att;
  //real mu_def;
  real<lower=0.5> c;
  real<lower=0.5> d;
  real gamma[3];
}
transformed parameters{
  vector[ngames] theta1;
  vector[ngames] theta2;
  vector[ngames] theta;
  vector[nteams] att;
  vector[nteams] def;
  //vector[nteams] alpha;
  
  for(t in 1:nteams){
    att[t]<-mu_att+c*prior_score_att[t]+b*prior_score[t]+
            sigma_att*e_att[t];
    def[t]<--d*prior_score_def[t]+sigma_def*e_def[t]-b*prior_score[t];
  }
  
  for (n in 1:ngames){
    theta1[n]<-exp(att[team1[n]]+def[team2[n]]+gamma[type_num[n]]);
    theta2[n]<-exp(att[team2[n]]+def[team1[n]]+gamma[type_num[n]]); 
    theta[n]<-theta1[n]-theta2[n]; 
  }
}
model{
  for (n in 1:ngames){
    increment_log_prob(poisson_log(score1[n],theta1[n]));
    increment_log_prob(poisson_log(score2[n],theta2[n]));
  }
  
  e_att~normal(0,1);
  e_def~normal(0,1);
  mu_att~normal(0,1);
  //mu_def~normal(0,1);
  sigma_att~cauchy(0,2.5);
  sigma_def~cauchy(0,2.5);
  //b~normal(0,1);
  //mu~normal(0,10);
  //c~uniform(0,10);
  //d~uniform(0,10);
  gamma~normal(0,1);
}
  
generated quantities{
  int score1_rep[ngames];
  int score2_rep[ngames];
  int diff_rep[ngames];
  
  int score1_prev[ngames_prev];
  int score2_prev[ngames_prev];
  int diff_prev[ngames_prev];
  
  vector[ngames_prev] theta1_prev;
  vector[ngames_prev] theta2_prev;
  vector[ngames_prev] theta_prev;
  
  
  for (n in 1:ngames){
    score1_rep[n]<-poisson_rng(theta1[n]);
    score2_rep[n]<-poisson_rng(theta2[n]);
    
    diff_rep[n]<-score1_rep[n]-score2_rep[n];
    
  }
  
  for (n in 1:ngames_prev){
    
    theta1_prev[n]<-exp(att[team1_prev[n]]+def[team2_prev[n]]+gamma[2]);
    theta2_prev[n]<-exp(att[team2_prev[n]]+def[team1_prev[n]]+gamma[2]);
    theta_prev[n]<-theta1_prev[n]-theta2_prev[n]; 
    
  }
  
  for (n in 1:ngames_prev){
    score1_prev[n]<-poisson_rng(theta1_prev[n]);
    score2_prev[n]<-poisson_rng(theta2_prev[n]);
    
    diff_prev[n]<-score1_prev[n]-score2_prev[n];
    
  }
  
}
